//**************************************************************
// MaxiCode Encoder for IDAutomation Barcode Fonts 2021 DEMO
//  Copyright, 2021 IDAutomation.com, Inc. All rights reserved.
// This encoder returns a text string as an Expression. This
// string will generate the barcode when combined with the
// correct barcode font.
// Font to use: IDAutomation Maxicode
//
//*****************************************************************
// Limitations of the Demo Version:
// The demo version of this product includes a fully
// functional 2D font and is designed only for the
// purpose of testing the font in various applications
// and point sizes. To generate dynamic symbols,
// a font encoder is required to format the data being
// encoded into the 2D symbol.
// Only static data can be provided for the encoders
// in the demo version, because providing a working
// encoder would reveal the complete source code,
// which is only available in the licensed version.
// To order, please visit:
// https://www.idautomation.com/
//*****************************************************************
//
// Please refer to the User Manual and Included Examples
// https://www.idautomation.com/barcode-fonts/2d/maxicode/
//
//*****************************************************************



using System;
using System.Linq;
using System.Text;

namespace IDAutomation.NetStandard.MaxiCode.FontEncoder
{
	/// <summary>
	/// Class constructor. 
	/// </summary>
	public class MaxiCode
	{
		#region Constants and tables
		internal const float HEX_SEPARATOR = .10F;
		private const int PAD = 33;
		private const int ECI = 27;
		private const int NSHIFT = 31;
		private const int SHIFTA = 59;
		private const int SHIFTB = 59;
		private const int SHIFTC = 60;
		private const int LOCKC = 60;
		private const int SHIFTD = 61;
		private const int LOCKD = 61;
		private const int SHIFTE = 62;
		private const int LOCKE = 62;
		private const int LACHTB = 63;
		private const int LACHTA_B = 63; // when in mode B
		private const int LACHTA = 58;
		private const int PADC = 55;
		private const int PADE = 29;
		private const int SHIFT2A = 56;
		private const int SHIFT3A = 57;

		private const char GS = (char) (29);
		private const char RS = (char) (30);

		/// <summary>layout of the hexagons at 8 dpm
		/// </summary>
		internal int[][] pattern8dpm = {new int[]{0, 0, 0, 0, 0, 0, 0},
										new int[]{0, 0, 0, 1, 1, 0, 0},
										new int[]{0, 0, 1, 1, 1, 1, 0},
										new int[]{0, 1, 1, 1, 1, 1, 1},
										new int[]{0, 1, 1, 1, 1, 1, 1},
										new int[]{0, 1, 1, 1, 1, 1, 1},
										new int[]{0, 0, 1, 1, 1, 1, 0},
										new int[]{0, 0, 0, 1, 1, 0, 0}};

		/// <summary>layout of the hexagons at 12 dpm
		/// </summary>
		internal int[][] pattern12dpm = {new int[]{0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
			                             new int[]{0, 0, 0, 0, 1, 1, 1, 0, 0, 0},
										 new int[]{0, 0, 0, 1, 1, 1, 1, 1, 0, 0},
										 new int[]{0, 0, 1, 1, 1, 1, 1, 1, 1, 0},
										 new int[]{0, 1, 1, 1, 1, 1, 1, 1, 1, 1},
										 new int[]{0, 1, 1, 1, 1, 1, 1, 1, 1, 1},
										 new int[]{0, 1, 1, 1, 1, 1, 1, 1, 1, 1},
										 new int[]{0, 1, 1, 1, 1, 1, 1, 1, 1, 1},
										 new int[]{0, 0, 1, 1, 1, 1, 1, 1, 1, 0},
										 new int[]{0, 0, 0, 1, 1, 1, 1, 1, 0, 0},
										 new int[]{0, 0, 0, 0, 1, 1, 1, 0, 0, 0},
										 new int[]{0, 0, 0, 0, 0, 0, 0, 0, 0, 0}};

		/// <summary>layout of the hexagons at 16 dpm
		/// </summary>
		internal int[][] pattern16dpm = {new int[]{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
										 new int[]{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
										 new int[]{0, 0, 0, 0, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0},
										 new int[]{0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 0, 0, 0},
										 new int[]{0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0},
										 new int[]{0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1},
										 new int[]{0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1},
										 new int[]{0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1},
										 new int[]{0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1},
										 new int[]{0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1},
										 new int[]{0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1},
										 new int[]{0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1},
										 new int[]{0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1},
										 new int[]{0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0},
										 new int[]{0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 0, 0, 0},
										 new int[]{0, 0, 0, 0, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0}};

		/// <summary>layout of the hexagons at 20 dpm
		/// </summary>
		internal int[][] pattern20dpm = {new int[]{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
									  	 new int[]{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
										 new int[]{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
										 new int[]{0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0, 0},
										 new int[]{0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0},
										 new int[]{0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0},
										 new int[]{0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0},
										 new int[]{0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1},
										 new int[]{0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1},
										 new int[]{0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1},
										 new int[]{0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1},
										 new int[]{0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1},
										 new int[]{0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1},
										 new int[]{0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1},
										 new int[]{0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1},
										 new int[]{0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1},
										 new int[]{0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0},
										 new int[]{0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0},
										 new int[]{0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0},
										 new int[]{0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0, 0}};
		
		/// <summary>layout of the hexagons at 24 dpm
		/// </summary>
		internal int[][] pattern24dpm = {new int[]{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
									 	 new int[]{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
										 new int[]{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
										 new int[]{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0},
										 new int[]{0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0},
										 new int[]{0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0},
										 new int[]{0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0},
										 new int[]{0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1},
										 new int[]{0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1},
										 new int[]{0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1},
										 new int[]{0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1},
										 new int[]{0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1},
										 new int[]{0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1},
										 new int[]{0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1},
										 new int[]{0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1},
										 new int[]{0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1},
										 new int[]{0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1},
										 new int[]{0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1},
										 new int[]{0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1},
										 new int[]{0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1},
										 new int[]{0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0},
										 new int[]{0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0},
										 new int[]{0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0},
										 new int[]{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0}};
				
		internal int[][] LookupTable = {new int[]{0, 'E', 0}, new int[]{1, 'E', 1}, new int[]{2, 'E', 2}, new int[]{3, 'E', 3}, 
										   new int[]{4, 'E', 4}, new int[]{5, 'E', 5}, new int[]{6, 'E', 6}, new int[]{7, 'E', 7}, 
										   new int[]{8, 'E', 8}, new int[]{9, 'E', 9}, new int[]{10, 'E', 10}, new int[]{11, 'E', 11}, 
										   new int[]{12, 'E', 12}, new int[]{13, 'E', 13}, new int[]{13, 'A', 0}, new int[]{14, 'E', 14},
										   new int[]{15, 'E', 15}, new int[]{16, 'E', 16}, new int[]{17, 'E', 17}, new int[]{18, 'E', 18}, 
										   new int[]{19, 'E', 19}, new int[]{20, 'E', 20}, new int[]{21, 'E', 21}, new int[]{22, 'E', 22}, 
										   new int[]{23, 'E', 23}, new int[]{24, 'E', 24}, new int[]{25, 'E', 25}, new int[]{26, 'E', 26}, 
										   new int[]{27, 'E', 30}, new int[]{28, 'E', 32}, new int[]{28, 'A', 28}, new int[]{28, 'B', 28}, 
										   new int[]{28, 'C', 28}, new int[]{28, 'D', 28}, new int[]{29, 'C', 29}, new int[]{29, 'D', 29}, 
										   new int[]{29, 'B', 29}, new int[]{29, 'A', 29}, new int[]{29, 'D', 33}, new int[]{30, 'C', 30}, 
										   new int[]{30, 'D', 30}, new int[]{30, 'B', 30}, new int[]{30, 'E', 34}, new int[]{30, 'A', 30}, 
										   new int[]{31, 'E', 35}, new int[]{32, 'D', 59}, new int[]{32, 'A', 32}, new int[]{32, 'E', 59}, 
										   new int[]{32, 'C', 59}, new int[]{32, 'B', 47}, new int[]{33, 'B', 53}, new int[]{34, 'A', 34}, 
										   new int[]{35, 'A', 35}, new int[]{36, 'A', 36}, new int[]{37, 'A', 37}, new int[]{38, 'A', 38}, 
										   new int[]{39, 'A', 39}, new int[]{40, 'A', 40}, new int[]{41, 'A', 41}, new int[]{42, 'A', 42}, 
										   new int[]{43, 'A', 43}, new int[]{44, 'A', 44}, new int[]{44, 'B', 48}, new int[]{45, 'A', 45}, 
										   new int[]{46, 'B', 49}, new int[]{46, 'A', 46}, new int[]{47, 'B', 50}, new int[]{47, 'A', 47}, 
										   new int[]{48, 'A', 48}, new int[]{49, 'A', 49}, new int[]{50, 'A', 50}, new int[]{51, 'A', 51}, 
										   new int[]{52, 'A', 52}, new int[]{53, 'A', 53}, new int[]{54, 'A', 54}, new int[]{55, 'A', 55}, 
										   new int[]{56, 'A', 56}, new int[]{57, 'A', 57}, new int[]{58, 'A', 58}, new int[]{58, 'B', 51}, 
										   new int[]{59, 'B', 37}, new int[]{60, 'B', 38}, new int[]{61, 'B', 39}, new int[]{62, 'B', 40}, 
										   new int[]{63, 'B', 41}, new int[]{64, 'B', 52}, new int[]{65, 'A', 1}, new int[]{66, 'A', 2}, 
										   new int[]{67, 'A', 3}, new int[]{68, 'A', 4}, new int[]{69, 'A', 5}, new int[]{70, 'A', 6}, 
										   new int[]{71, 'A', 7}, new int[]{72, 'A', 8}, new int[]{73, 'A', 9}, new int[]{74, 'A', 10}, 
										   new int[]{75, 'A', 11}, new int[]{76, 'A', 12}, new int[]{77, 'A', 13}, new int[]{78, 'A', 14}, 
										   new int[]{79, 'A', 15}, new int[]{80, 'A', 16}, new int[]{81, 'A', 17}, new int[]{82, 'A', 18}, 
										   new int[]{83, 'A', 19}, new int[]{84, 'A', 20}, new int[]{85, 'A', 21}, new int[]{86, 'A', 22}, 
										   new int[]{87, 'A', 23}, new int[]{88, 'A', 24}, new int[]{89, 'A', 25}, new int[]{90, 'A', 26}, 
										   new int[]{91, 'B', 42}, new int[]{92, 'B', 43}, new int[]{93, 'B', 44}, new int[]{94, 'B', 45}, 
										   new int[]{95, 'B', 46}, new int[]{96, 'B', 0}, new int[]{97, 'B', 1}, new int[]{98, 'B', 2}, 
										   new int[]{99, 'B', 3}, new int[]{100, 'B', 4}, new int[]{101, 'B', 5}, new int[]{102, 'B', 6}, 
										   new int[]{103, 'B', 7}, new int[]{104, 'B', 8}, new int[]{105, 'B', 9}, new int[]{106, 'B', 10}, 
										   new int[]{107, 'B', 11}, new int[]{108, 'B', 12}, new int[]{109, 'B', 13}, new int[]{110, 'B', 14},
										   new int[]{111, 'B', 15}, new int[]{112, 'B', 16}, new int[]{113, 'B', 17}, new int[]{114, 'B', 18}, 
										   new int[]{115, 'B', 19}, new int[]{116, 'B', 20}, new int[]{117, 'B', 21}, new int[]{118, 'B', 22}, 
										   new int[]{119, 'B', 23}, new int[]{120, 'B', 24}, new int[]{121, 'B', 25}, new int[]{122, 'B', 26}, 
										   new int[]{123, 'B', 32}, new int[]{124, 'B', 54}, new int[]{125, 'B', 34}, new int[]{126, 'B', 35}, 
										   new int[]{127, 'B', 36}, new int[]{128, 'C', 48}, new int[]{129, 'C', 49}, new int[]{130, 'C', 50}, 
										   new int[]{131, 'C', 51}, new int[]{132, 'C', 52}, new int[]{133, 'C', 53}, new int[]{134, 'C', 54}, 
										   new int[]{135, 'C', 55}, new int[]{136, 'C', 56}, new int[]{137, 'C', 57}, new int[]{138, 'D', 47}, 
										   new int[]{139, 'D', 48}, new int[]{140, 'D', 49}, new int[]{141, 'D', 50}, new int[]{142, 'D', 51}, 
										   new int[]{143, 'D', 52}, new int[]{144, 'D', 53}, new int[]{145, 'D', 54}, new int[]{146, 'D', 55}, 
										   new int[]{147, 'D', 56}, new int[]{148, 'D', 57}, new int[]{149, 'E', 48}, new int[]{150, 'E', 49},
										   new int[]{151, 'E', 50}, new int[]{152, 'E', 51}, new int[]{153, 'E', 52}, new int[]{154, 'E', 53},
										   new int[]{155, 'E', 54}, new int[]{156, 'E', 55}, new int[]{157, 'E', 56}, new int[]{158, 'E', 57},
										   new int[]{159, 'E', 36}, new int[]{160, 'E', 37}, new int[]{161, 'D', 37}, new int[]{162, 'E', 38},
										   new int[]{163, 'E', 39}, new int[]{164, 'E', 40}, new int[]{165, 'E', 41}, new int[]{166, 'D', 42},
										   new int[]{167, 'E', 43}, new int[]{168, 'D', 38}, new int[]{169, 'E', 44}, new int[]{170, 'C', 37}, 
										   new int[]{171, 'D', 39}, new int[]{172, 'C', 38}, new int[]{173, 'E', 45}, new int[]{174, 'E', 46}, 
										   new int[]{175, 'D', 40}, new int[]{176, 'D', 41}, new int[]{177, 'C', 39}, new int[]{178, 'C', 40}, 
										   new int[]{179, 'C', 41}, new int[]{180, 'D', 42}, new int[]{181, 'C', 42}, new int[]{182, 'E', 47}, 
										   new int[]{183, 'D', 43}, new int[]{184, 'D', 44}, new int[]{185, 'C', 43}, new int[]{186, 'C', 44}, 
										   new int[]{187, 'D', 45}, new int[]{188, 'C', 45}, new int[]{189, 'C', 46}, new int[]{190, 'C', 47},
										   new int[]{191, 'D', 46}, new int[]{192, 'C', 0}, new int[]{193, 'C', 1}, new int[]{194, 'C', 2}, 
										   new int[]{195, 'C', 3}, new int[]{196, 'C', 4}, new int[]{197, 'C', 5}, new int[]{198, 'C', 6}, 
										   new int[]{199, 'C', 7}, new int[]{200, 'C', 8}, new int[]{201, 'C', 9}, new int[]{202, 'C', 10},
										   new int[]{203, 'C', 11}, new int[]{204, 'C', 12}, new int[]{205, 'C', 13}, new int[]{206, 'C', 14},
										   new int[]{207, 'C', 15}, new int[]{208, 'C', 16}, new int[]{209, 'C', 17}, new int[]{210, 'C', 18},
										   new int[]{211, 'C', 19}, new int[]{212, 'C', 20}, new int[]{213, 'C', 21}, new int[]{214, 'C', 22}, 
										   new int[]{215, 'C', 23}, new int[]{216, 'C', 24}, new int[]{217, 'C', 25}, new int[]{218, 'C', 26}, 
										   new int[]{219, 'C', 32}, new int[]{220, 'C', 33}, new int[]{221, 'C', 34}, new int[]{222, 'C', 35},
										   new int[]{223, 'C', 36}, new int[]{224, 'D', 0}, new int[]{225, 'D', 1}, new int[]{226, 'D', 2},
										   new int[]{227, 'D', 3}, new int[]{228, 'D', 4}, new int[]{229, 'D', 5}, new int[]{230, 'D', 6}, 
										   new int[]{231, 'D', 7}, new int[]{232, 'D', 8}, new int[]{233, 'D', 9}, new int[]{234, 'D', 10}, 
										   new int[]{235, 'D', 11}, new int[]{236, 'D', 12}, new int[]{237, 'D', 13}, new int[]{238, 'D', 14},
										   new int[]{239, 'D', 15}, new int[]{240, 'D', 16}, new int[]{241, 'D', 17}, new int[]{242, 'D', 18},
										   new int[]{243, 'D', 19}, new int[]{244, 'D', 20}, new int[]{245, 'D', 21}, new int[]{246, 'D', 22},
										   new int[]{247, 'D', 23}, new int[]{248, 'D', 24}, new int[]{249, 'D', 25}, new int[]{250, 'D', 26},
										   new int[]{251, 'D', 32}, new int[]{252, 'C', 33}, new int[]{253, 'D', 34}, new int[]{254, 'D', 35}, 
										   new int[]{255, 'D', 36}};
		#endregion


		/// <summary>
		/// Function, which creates a MaxiCode barcode, depending on the various input options chosen. 
		/// </summary>
		/// <param name="DataToEncode">The data that is encoded in the barcode.</param>
		/// <param name="ApplyTilde">Enables processing of tilde and parentheses characters for purposes of encoding ASCII functions and GS1 application identifiers.</param>
		/// <param name="EncodingMode">Various modes used to Data represented in the symbol may be encoded by one of these types.</param>
		/// <param name="QuietZone">Define the size of a blank zone around the code.</param>
		/// <param name="DotPM">Define the size of each module.</param>
		/// <returns>A byte stream representation of vector Maxicode's raster image.</returns>
		public byte[] EncodeMaxiCode(string DataToEncode,
			                         bool ApplyTilde = true,
									 EncodingModes EncMode = EncodingModes.Mode2,
									 int QuietZone = 2,
									 int DotPM = 16)
		{
			encodingMode = EncMode;
			_encode(DataToEncode, ApplyTilde); //QuietZone);

			byte[] Result = null;

			int k;
			int intPad = 0;
			int pixels = 0;
			int bit = 0;
			int bWidth = 0;
			int rows = 33;
			int cols = 30;

			int ModuleSizeX = 0;
			int ModuleSizeY = 0;


			int SizeX = 19;
			int SizeY = 17;
			double Ray = 16d * DotPM;

			int[][] dots = pattern24dpm;
			if (DotPM == 20)
			{
				dots = pattern20dpm;
				SizeX = 16;
				SizeY = 14;
				Ray = 14d * DotPM;

			}
			else if (DotPM == 16)
			{
				dots = pattern16dpm;
				SizeX = 14;
				SizeY = 12;
				Ray = 13d * DotPM;
			}
			else if (DotPM == 12)
			{
				dots = pattern12dpm;
				SizeX = 11;
				SizeY = 9;
				Ray = 11d * DotPM;
			}
			else if (DotPM == 8)
			{
				dots = pattern8dpm;
				SizeX = 7;
				SizeY = 6;
				Ray = 7d * DotPM;

			}
			ModuleSizeY = dots.Length;
			ModuleSizeX = dots[0].Length;

			if (QuietZone < 0) QuietZone = 0;

			int OffsetX = SizeX/2;
//			int OffsetY = ModuleSize * 1.5d;

			int[,] bitmap = new int[(rows+1+ 2* QuietZone)*SizeY , (cols+1+2* QuietZone)*SizeX];

			for (int y = 0; y < rows; y++)
			{
				for (int x = 0; x < cols; x++)
				{
					if ((y & 1) != 0)
						OffsetX = SizeX/2;
					else
						OffsetX = 0;

					if (Matrix[y][x] == 1)
					{

						for (int n = 0; n < ModuleSizeY; n++)
						{
							for (int m = 0; m < ModuleSizeX; m++)
							{
								if (dots[n][m] == 1)
                                {
									bitmap[(SizeY * (y + QuietZone))+ n , (SizeX * (x + QuietZone)) + m + OffsetX ] = 1;
                                }

							}

						}


					}
				}
			}

			int Cx = (14 + QuietZone) * SizeX + SizeX / 2;
			int Cy = (16 + QuietZone) * SizeY + SizeY / 2;
			double cat = 0;

			for (int y = (int)(Cy - 6* SizeY); y < (Cy + (6 * SizeY)); y++)
			{
				for (int x = (int)(Cx -(6 * SizeX)); x < (Cx + (6 * SizeX)); x++)
				{
					cat = Math.Pow(x - Cx, 2) + Math.Pow(y - Cy, 2);
					if ((cat > Ray * 12 & cat <= Ray * 18) | (cat > Ray * 4 & cat <= Ray * 8) | (cat > 0.4 * Ray & cat <= Ray * 1.5))
					{
						bitmap[y , x ] = 1;
					}
				}
			}

			cols = cols + 2 * QuietZone;
			rows = rows + 2 * QuietZone;


			bWidth = ((int)((cols * SizeX) / 8) + ((cols * SizeX) % 8 > 0 ? 1 : 0));
			int intTemp = bWidth % 4;
			if (intTemp != 0)
			{
				intPad = 4 - intTemp;
			}

			Result = new byte[64 + (rows+1) * SizeY * (bWidth + intPad)];

			Result[0] = 0x42;
			Result[1] = 0x4d;
			for (k = 0; k < 8; k++)
			{
				Result[2 + k] = 0;
			}
			Result[10] = 0x3E;
			for (k = 0; k < 3; k++)
			{
				Result[11 + k] = 0;
			}
			Result[14] = 0x28;
			for (k = 0; k < 3; k++)
			{
				Result[15 + k] = 0;
			}

			// width
			Result[18] = (byte)((cols * SizeX) % 256);
			Result[19] = (byte)(Math.Floor((double)(cols * SizeX) / 256));
			Result[20] = 0;
			Result[21] = 0;

			// height
			Result[22] = (byte)(((rows+1) * SizeY) % 256);
			Result[23] = (byte)(Math.Floor((double)((rows+1) * SizeY) / 256));
			Result[24] = 0; ;
			Result[25] = 0;

			Result[26] = 1;
			Result[27] = 0;
			Result[28] = 1;
			Result[29] = 0;
			for (k = 0; k < 8; k++)
			{
				Result[30 + k] = 0;
			}
			Result[38] = 0x25;
			Result[39] = 0x0e;
			Result[40] = 0;
			Result[41] = 0;

			Result[42] = 0x25;
			Result[43] = 0x0e;
			Result[44] = 0;
			Result[45] = 0;

			for (k = 0; k < 8; k++)
			{
				Result[46 + k] = 0;
			}

			for (k = 0; k < 3; k++)
			{
				Result[54 + k] = 0xff;
			}
			for (k = 0; k < 5; k++)
			{
				Result[57 + k] = 0;
			}

			int pos = 62;
			for (int y = (rows+1)*SizeY - 1; y >= 0; y--)
			{

					pixels = 0;
					bit = 8;

					for (int x = 0; x < (cols*SizeX); x++)
					{

							bit = bit - 1;
							if (bitmap[y, x] == 1)  pixels = pixels + (int)Math.Pow(2, bit);
							if (bit == 0)
							{
								Result[pos] = (byte)pixels;
								pos++;
								bit = 8;
								pixels = 0;
							}
					}
					if (bit != 8)
					{
						Result[pos] = (byte)pixels;
						pos++;
					}
					pos = pos + intPad;

			}

			return Result;


		}


		/// <summary>
		/// Function, which creates a Maxicode barcode, depending on the various input options chosen. 
		/// </summary>
		/// <param name="DataToEncode">The data that is encoded in the barcode.</param>
		/// <param name="ApplyTilde">Enables processing of tilde and parentheses characters for purposes of encoding ASCII functions and GS1 application identifiers.</param>
		/// <param name="EncodingMode">Various modes used to Data represented in the symbol may be encoded by one of these types.</param>
		/// <param name="VectorType">Define the type of vector image for output.</param>
		/// <param name="NewLine">String indicating EOL character. Defaults to the operating system EOL character if left NULL.</param>
		/// <param name="QuietZone">Define the size of a blank zone around the code.</param>
		/// <param name="DotPM">Define the size of each module.</param>
		/// <returns>A string representation of vector Maxicode's image.</returns>
		public string EncodeMaxiCode(string DataToEncode, bool ApplyTilde = true, EncodingModes EncMode = EncodingModes.Mode2, VectorTypes VectorType = VectorTypes.VecEPS, string NewLine = null, int QuietZone = 2, int DotPM = 16)
		{

			encodingMode = EncMode;
			//allows user to set an end-of-line (EOL) string
			if (NewLine == null)
			{
				NewLine = Environment.NewLine; //default
			}

			_newline = NewLine;

			_encode(DataToEncode, ApplyTilde); //, QuietZone);

			int rows = 33;
			int cols = 30;

			StringBuilder strOut = new StringBuilder("");
			string tempW = "";
			string tempH = "";


			double ModuleSize = 4;

			if (DotPM == 20)
				ModuleSize = 3.5;
			else if (DotPM == 16)
				ModuleSize = 3;
			else if (DotPM == 12)
				ModuleSize = 2.1;
			else if (DotPM == 8)
				ModuleSize = 1.5d;

			if (QuietZone < 0) QuietZone = 0;

			Double ModuleSizeX = ModuleSize * 4.9;
			Double ModuleSizeY = ModuleSize * 4.1; 

			double Ray = 0.4d*ModuleSizeX; 
			double OffsetX = ModuleSizeX *0.5d;
			double OffsetY = ModuleSizeY *0.5d;

			tempW = ((cols + 0.5 + 2* QuietZone) * ModuleSizeX + OffsetX).ToString().Replace(",", ".");
			tempH = ((rows+ 2* QuietZone) * ModuleSizeY + OffsetY ).ToString().Replace(",", ".");

			if (VectorType == VectorTypes.VecSVG)
			{
				OffsetY = 0.5d*ModuleSizeY;

				strOut.Append("<?xml version='1.0' standalone='no'?>" + _newline);
				strOut.Append("<!DOCTYPE svg PUBLIC '-//W3C//DTD SVG 1.1//EN' 'http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd'>" + _newline);
				strOut.Append("<svg width='" + tempW + "' height='" + tempH + "' version='1.1' xmlns='http://www.w3.org/2000/svg'>" + _newline);
				strOut.Append("  <g id='maxicode'>" + _newline);
				strOut.Append("    <g id='background' fill='#ffffff' stroke='#ffffff'>" + _newline);
				strOut.Append("      <rect x='0' y='0' width='" + tempW + "' height='" + tempH + "' />" + _newline);
				strOut.Append("    </g>" + _newline);
				strOut.Append("    <g id='pattern' fill='#000000' stroke='#000000'>" + _newline);

				for (int y = 0; y < rows; y++)
					{
					for (int x = 0; x < cols; x++)
					{
						if ((y & 1) != 0)
							OffsetX = ModuleSizeX;
						else
							OffsetX = ModuleSizeX*0.5;
						if (Matrix[y][x] == 1) 
						{
							strOut.Append("    <g id='R" + y + "C" + x + "' >" + _newline);
							strOut.Append("      <polygon points='" +
												((((x+QuietZone) * ModuleSizeX) + Ray * 0.891 + OffsetX).ToString()).Replace(",", ".") + "," + ((((y + QuietZone) * ModuleSizeY) + OffsetY - Ray * 0.454).ToString()).Replace(",", ".") + " " +
												((((x + QuietZone) * ModuleSizeX) + OffsetX).ToString()).Replace(",", ".") + "," + ((((y + QuietZone) * ModuleSizeY) + OffsetY - Ray).ToString()).Replace(",", ".") + " " +
												((((x + QuietZone) * ModuleSizeX) - Ray * 0.891 + OffsetX).ToString()).Replace(",", ".") + "," + ((((y + QuietZone) * ModuleSizeY) + OffsetY - Ray * 0.454).ToString()).Replace(",", ".") + " " +
												((((x + QuietZone) * ModuleSizeX) - Ray * 0.891 + OffsetX).ToString()).Replace(",", ".") + "," + ((((y + QuietZone) * ModuleSizeY) + OffsetY + Ray * 0.454).ToString()).Replace(",", ".") + " " +
												((((x + QuietZone) * ModuleSizeX) + OffsetX).ToString()).Replace(",", ".") + "," + ((((y + QuietZone) * ModuleSizeY) + OffsetY + Ray).ToString()).Replace(",", ".") + " " +
												((((x + QuietZone) * ModuleSizeX) + Ray * 0.891 + OffsetX).ToString()).Replace(",", ".") + "," + ((((y + QuietZone) * ModuleSizeY) + OffsetY + Ray * 0.454).ToString()).Replace(",", ".") + " " +
                                "' />" + NewLine);
							strOut.Append("    </g>" + _newline);
						}
					}
				}
				strOut.Append("    </g>" + _newline);
				int cx = 14 + QuietZone;
				int cy = 16 + QuietZone;
				OffsetX = ModuleSizeX * 0.5d;
				Ray = 0.58d * ModuleSizeX;
				strOut.Append("    <g id='target'>" + _newline);
				strOut.Append(("      <circle style='fill:#000000; stroke:none;' cx='" + (cx  * ModuleSizeX + OffsetX).ToString() + "' cy='" + (cy * ModuleSizeY + OffsetY).ToString() + "' r='" + (Ray * 7.59).ToString() + "' />").Replace(",", ".") + NewLine);
				strOut.Append(("      <circle style='fill:#ffffff; stroke:none;' cx='" + (cx  * ModuleSizeX + OffsetX).ToString() + "' cy='" + (cy * ModuleSizeY + OffsetY).ToString() + "' r='" + (Ray * 6.27).ToString() + "' />").Replace(",", ".") + NewLine);
				strOut.Append(("      <circle style='fill:#000000; stroke:none;' cx='" + (cx  * ModuleSizeX + OffsetX).ToString() + "' cy='" + (cy * ModuleSizeY + OffsetY).ToString() + "' r='" + (Ray * 4.96).ToString() + "' />").Replace(",", ".") + NewLine);
				strOut.Append(("      <circle style='fill:#ffffff; stroke:none;' cx='" + (cx  * ModuleSizeX + OffsetX).ToString() + "' cy='" + (cy * ModuleSizeY + OffsetY).ToString() + "' r='" + (Ray * 3.64).ToString() + "' />").Replace(",", ".") + NewLine);
				strOut.Append(("      <circle style='fill:#000000; stroke:none;' cx='" + (cx  * ModuleSizeX + OffsetX).ToString() + "' cy='" + (cy * ModuleSizeY + OffsetY).ToString() + "' r='" + (Ray * 2.31).ToString() + "' />").Replace(",", ".") + NewLine);
				strOut.Append(("      <circle style='fill:#ffffff; stroke:none;' cx='" + (cx  * ModuleSizeX + OffsetX).ToString() + "' cy='" + (cy * ModuleSizeY + OffsetY).ToString() + "' r='" + (Ray).ToString() + "' />").Replace(",", ".") + NewLine);
				strOut.Append("    </g>" + _newline);
				strOut.Append("  </g>" + _newline);
				strOut.Append("</svg>" + _newline);
			}
			else if (VectorType == VectorTypes.VecEPS)
			{

                strOut.Append("%!PS-Adobe-3.0 EPSF-3.0" + _newline);
                strOut.Append("%%Creator: IDAutomation.com" + _newline);
                strOut.Append("%%Title: maxicode" + _newline);
                strOut.Append("%%CreationDate: " + DateTime.Now.ToString("ddd mmm d hh:mm:ss yyyy") + _newline);
                strOut.Append("%%BoundingBox: 0 0 " + tempW + " " + tempH + _newline);
                strOut.Append("%%DocumentData: Clean7Bit" + _newline);
                strOut.Append("%%EndComments" + _newline);
                strOut.Append("%%BeginProlog" + _newline);
                strOut.Append("/bd { bind def } bind def" + _newline);
                strOut.Append("/incompound false def" + _newline);
                strOut.Append("/m { moveto } bd" + _newline);
                strOut.Append("/l { lineto } bd" + _newline);
                strOut.Append("/*U { /incompound false def f} bd" + _newline);
                strOut.Append("/k { setcmykcolor } bd" + _newline);

                strOut.Append("/c { curveto } bd" + _newline);
                strOut.Append("/F { incompound not {fill} if } bd" + _newline);
                strOut.Append("/f { closepath F } bd" + _newline);
                strOut.Append("/S { stroke } bd" + _newline);
                strOut.Append("/*u { /incompound true def } bd" + _newline);
                strOut.Append("/K { k } bd" + _newline);

                strOut.Append("%%EndProlog" + _newline);
                strOut.Append("%%BeginSetup" + _newline);
                strOut.Append("%%EndSetup" + _newline);
                strOut.Append("0.000 0.000 0.000 0.000 k" + _newline);
                strOut.Append("0 0 m" + _newline);
                strOut.Append("0 0 l" + _newline);
                strOut.Append(tempW + " 0 l" + _newline);
                strOut.Append(tempW + " " + tempH + " l" + _newline);
                strOut.Append("0 " + tempH + " l" + _newline);
                strOut.Append("f" + _newline);
                strOut.Append("*U" + _newline);

				ModuleSizeX = ModuleSize * 4.9;
				ModuleSizeY = ModuleSize * 4.1;
				Ray = 0.5d * ModuleSizeX;
				//int n = rows * ModuleSizeY;
				strOut.Append("0.000 0.000 0.000 1.000 k" + _newline);
                for (int y = 0; y < rows; y++)
                {
                    for (int x = 0; x < cols; x++)
                    {

						if ((y & 1) != 0)
							OffsetX = ModuleSizeX;
						else
							OffsetX = ModuleSizeX*0.5;

						if (Matrix[rows - 1 - y][x] == 1)
                        {
							strOut.Append((( ((x + QuietZone) * ModuleSizeX) + Ray * 0.891 + OffsetX).ToString()).Replace(",", ".") + " " + ((((y + QuietZone) * ModuleSizeY) + OffsetY - Ray * 0.454).ToString()).Replace(",", ".") + " m" + _newline);
							strOut.Append(((((x + QuietZone) * ModuleSizeX) + Ray * 0.891 + OffsetX).ToString()).Replace(",", ".") + " " + ((((y + QuietZone) * ModuleSizeY) + OffsetY - Ray * 0.454).ToString()).Replace(",", ".") + " l" + _newline);
							strOut.Append(((((x + QuietZone) * ModuleSizeX) + OffsetX).ToString()).Replace(",", ".") + " " + ((((y + QuietZone) * ModuleSizeY) + OffsetY - Ray).ToString()).Replace(",", ".") + " l" + _newline);
							strOut.Append(((((x + QuietZone) * ModuleSizeX) - Ray * 0.891 + OffsetX).ToString()).Replace(",", ".") + " " + ((((y + QuietZone) * ModuleSizeY) + OffsetY - Ray * 0.454).ToString()).Replace(",", ".") + " l" + _newline);
							strOut.Append(((((x + QuietZone) * ModuleSizeX) - Ray * 0.891 + OffsetX).ToString()).Replace(",", ".") + " " + ((((y + QuietZone) * ModuleSizeY) + OffsetY + Ray * 0.454).ToString()).Replace(",", ".") + " l" + _newline);
							strOut.Append(((((x + QuietZone) * ModuleSizeX) + OffsetX).ToString()).Replace(",", ".") + " " + ((((y + QuietZone) * ModuleSizeY) + OffsetY + Ray).ToString()).Replace(",", ".") + " l" + _newline);
							strOut.Append(((((x + QuietZone) * ModuleSizeX) + Ray * 0.891 + OffsetX).ToString()).Replace(",", ".") + " " + ((((y + QuietZone) * ModuleSizeY) + OffsetY + Ray * 0.454).ToString()).Replace(",", ".") + " l" + _newline);
							strOut.Append("f" + _newline);
                        }
                    }
                }


				double sngL;

				OffsetX = ModuleSizeX * 0.5;
				OffsetY = ModuleSizeX * 0.5;
				int Cx = 14 + QuietZone;
				int Cy = 16 + QuietZone;

				Ray = 0.57d * ModuleSizeX * 7.59; 
				sngL = (double)(4d / 3d * (Math.Sqrt(2d) - 1d) * Ray);

				strOut.Append("0.000 0.000 0.000 1.000 k" + _newline);
				strOut.Append((  (Cx * ModuleSizeX + OffsetX + Ray).ToString() + " " + (Cy* ModuleSizeY + OffsetY).ToString() + " m").Replace(",", ".") + NewLine);

				strOut.Append((  (Cx * ModuleSizeX + OffsetX + Ray).ToString() + " " + (Cy * ModuleSizeY + OffsetY - sngL).ToString() + " " + 
									(Cx * ModuleSizeX + OffsetX + sngL).ToString() + " " + (Cy * ModuleSizeY + OffsetY - Ray).ToString() + " " + 
									(Cx * ModuleSizeX + OffsetX).ToString() + " " + (Cy * ModuleSizeY + OffsetY - Ray).ToString() + " c").Replace(",", ".") + NewLine);

				strOut.Append((  (Cx * ModuleSizeX + OffsetX - sngL).ToString() + " " + (Cy * ModuleSizeY + OffsetY - Ray).ToString() + " " + 
									(Cx * ModuleSizeX + OffsetX - Ray).ToString() + " " + (Cy * ModuleSizeY + OffsetY - sngL).ToString() + " " + 
									(Cx * ModuleSizeX + OffsetX - Ray).ToString() + " " + (Cy * ModuleSizeY + OffsetY).ToString() + " c").Replace(",", ".") + NewLine);

				strOut.Append((  (Cx * ModuleSizeX + OffsetX - Ray).ToString() + " " + (Cy * ModuleSizeY + OffsetY + sngL).ToString() + " " + 
									(Cx * ModuleSizeX + OffsetX - sngL).ToString() + " " + (Cy * ModuleSizeY + OffsetY + Ray).ToString() + " " + 
									(Cx * ModuleSizeX + OffsetX).ToString() + " " + (Cy * ModuleSizeY + OffsetY + Ray).ToString() + " c").Replace(",", ".") + NewLine);

				strOut.Append((  (Cx * ModuleSizeX + OffsetX + sngL).ToString() + " " + (Cy * ModuleSizeY + OffsetY + Ray).ToString() + " " + 
									(Cx * ModuleSizeX + OffsetX + Ray).ToString() + " " + (Cy * ModuleSizeY + OffsetY + sngL).ToString() + " " + 
									(Cx * ModuleSizeX + OffsetX + Ray).ToString() + " " + (Cy * ModuleSizeY + OffsetY).ToString() + " c").Replace(",", ".") + NewLine);
				strOut.Append("f" + NewLine);

				Ray = 0.57d * ModuleSizeX * 6.27;
				sngL = (double)(4d / 3d * (Math.Sqrt(2d) - 1d) * Ray);
				strOut.Append("0.000 0.000 0.000 0.000 k" + _newline);
				strOut.Append(((Cx * ModuleSizeX + OffsetX + Ray).ToString() + " " + (Cy * ModuleSizeY + OffsetY).ToString() + " m").Replace(",", ".") + NewLine);

				strOut.Append(((Cx * ModuleSizeX + OffsetX + Ray).ToString() + " " + (Cy * ModuleSizeY + OffsetY - sngL).ToString() + " " +
								 (Cx * ModuleSizeX + OffsetX + sngL).ToString() + " " + (Cy * ModuleSizeY + OffsetY - Ray).ToString() + " " +
								 (Cx * ModuleSizeX + OffsetX).ToString() + " " + (Cy * ModuleSizeY + OffsetY - Ray).ToString() + " c").Replace(",", ".") + NewLine);

				strOut.Append(((Cx * ModuleSizeX + OffsetX - sngL).ToString() + " " + (Cy * ModuleSizeY + OffsetY - Ray).ToString() + " " +
								 (Cx * ModuleSizeX + OffsetX - Ray).ToString() + " " + (Cy * ModuleSizeY + OffsetY - sngL).ToString() + " " +
								 (Cx * ModuleSizeX + OffsetX - Ray).ToString() + " " + (Cy * ModuleSizeY + OffsetY).ToString() + " c").Replace(",", ".") + NewLine);

				strOut.Append(((Cx * ModuleSizeX + OffsetX - Ray).ToString() + " " + (Cy * ModuleSizeY + OffsetY + sngL).ToString() + " " +
								 (Cx * ModuleSizeX + OffsetX - sngL).ToString() + " " + (Cy * ModuleSizeY + OffsetY + Ray).ToString() + " " +
								 (Cx * ModuleSizeX + OffsetX).ToString() + " " + (Cy * ModuleSizeY + OffsetY + Ray).ToString() + " c").Replace(",", ".") + NewLine);

				strOut.Append(((Cx * ModuleSizeX + OffsetX + sngL).ToString() + " " + (Cy * ModuleSizeY + OffsetY + Ray).ToString() + " " +
								 (Cx * ModuleSizeX + OffsetX + Ray).ToString() + " " + (Cy * ModuleSizeY + OffsetY + sngL).ToString() + " " +
								 (Cx * ModuleSizeX + OffsetX + Ray).ToString() + " " + (Cy * ModuleSizeY + OffsetY).ToString() + " c").Replace(",", ".") + NewLine);
				strOut.Append("f" + NewLine);

				Ray = 0.57d * ModuleSizeX * 4.96;
				sngL = (double)(4d / 3d * (Math.Sqrt(2d) - 1d) * Ray);
				strOut.Append("0.000 0.000 0.000 1.000 k" + _newline);
				strOut.Append(((Cx * ModuleSizeX + OffsetX + Ray).ToString() + " " + (Cy * ModuleSizeY + OffsetY).ToString() + " m").Replace(",", ".") + NewLine);

				strOut.Append(((Cx * ModuleSizeX + OffsetX + Ray).ToString() + " " + (Cy * ModuleSizeY + OffsetY - sngL).ToString() + " " +
								 (Cx * ModuleSizeX + OffsetX + sngL).ToString() + " " + (Cy * ModuleSizeY + OffsetY - Ray).ToString() + " " +
								 (Cx * ModuleSizeX + OffsetX).ToString() + " " + (Cy * ModuleSizeY + OffsetY - Ray).ToString() + " c").Replace(",", ".") + NewLine);

				strOut.Append(((Cx * ModuleSizeX + OffsetX - sngL).ToString() + " " + (Cy * ModuleSizeY + OffsetY - Ray).ToString() + " " +
								 (Cx * ModuleSizeX + OffsetX - Ray).ToString() + " " + (Cy * ModuleSizeY + OffsetY - sngL).ToString() + " " +
								 (Cx * ModuleSizeX + OffsetX - Ray).ToString() + " " + (Cy * ModuleSizeY + OffsetY).ToString() + " c").Replace(",", ".") + NewLine);

				strOut.Append(((Cx * ModuleSizeX + OffsetX - Ray).ToString() + " " + (Cy * ModuleSizeY + OffsetY + sngL).ToString() + " " +
								 (Cx * ModuleSizeX + OffsetX - sngL).ToString() + " " + (Cy * ModuleSizeY + OffsetY + Ray).ToString() + " " +
								 (Cx * ModuleSizeX + OffsetX).ToString() + " " + (Cy * ModuleSizeY + OffsetY + Ray).ToString() + " c").Replace(",", ".") + NewLine);

				strOut.Append(((Cx * ModuleSizeX + OffsetX + sngL).ToString() + " " + (Cy * ModuleSizeY + OffsetY + Ray).ToString() + " " +
								 (Cx * ModuleSizeX + OffsetX + Ray).ToString() + " " + (Cy * ModuleSizeY + OffsetY + sngL).ToString() + " " +
								 (Cx * ModuleSizeX + OffsetX + Ray).ToString() + " " + (Cy * ModuleSizeY + OffsetY).ToString() + " c").Replace(",", ".") + NewLine);
				strOut.Append("f" + NewLine);


				Ray = 0.57d * ModuleSizeX * 3.64;
				sngL = (double)(4d / 3d * (Math.Sqrt(2d) - 1d) * Ray);
				strOut.Append("0.000 0.000 0.000 0.000 k" + _newline);
				strOut.Append(((Cx * ModuleSizeX + OffsetX + Ray).ToString() + " " + (Cy * ModuleSizeY + OffsetY).ToString() + " m").Replace(",", ".") + NewLine);

				strOut.Append(((Cx * ModuleSizeX + OffsetX + Ray).ToString() + " " + (Cy * ModuleSizeY + OffsetY - sngL).ToString() + " " +
								 (Cx * ModuleSizeX + OffsetX + sngL).ToString() + " " + (Cy * ModuleSizeY + OffsetY - Ray).ToString() + " " +
								 (Cx * ModuleSizeX + OffsetX).ToString() + " " + (Cy * ModuleSizeY + OffsetY - Ray).ToString() + " c").Replace(",", ".") + NewLine);

				strOut.Append(((Cx * ModuleSizeX + OffsetX - sngL).ToString() + " " + (Cy * ModuleSizeY + OffsetY - Ray).ToString() + " " +
								 (Cx * ModuleSizeX + OffsetX - Ray).ToString() + " " + (Cy * ModuleSizeY + OffsetY - sngL).ToString() + " " +
								 (Cx * ModuleSizeX + OffsetX - Ray).ToString() + " " + (Cy * ModuleSizeY + OffsetY).ToString() + " c").Replace(",", ".") + NewLine);

				strOut.Append(((Cx * ModuleSizeX + OffsetX - Ray).ToString() + " " + (Cy * ModuleSizeY + OffsetY + sngL).ToString() + " " +
								 (Cx * ModuleSizeX + OffsetX - sngL).ToString() + " " + (Cy * ModuleSizeY + OffsetY + Ray).ToString() + " " +
								 (Cx * ModuleSizeX + OffsetX).ToString() + " " + (Cy * ModuleSizeY + OffsetY + Ray).ToString() + " c").Replace(",", ".") + NewLine);

				strOut.Append(((Cx * ModuleSizeX + OffsetX + sngL).ToString() + " " + (Cy * ModuleSizeY + OffsetY + Ray).ToString() + " " +
								 (Cx * ModuleSizeX + OffsetX + Ray).ToString() + " " + (Cy * ModuleSizeY + OffsetY + sngL).ToString() + " " +
								 (Cx * ModuleSizeX + OffsetX + Ray).ToString() + " " + (Cy * ModuleSizeY + OffsetY).ToString() + " c").Replace(",", ".") + NewLine);
				strOut.Append("f" + NewLine);


				Ray = 0.57d * ModuleSizeX * 2.31;
				sngL = (double)(4d / 3d * (Math.Sqrt(2d) - 1d) * Ray);
				strOut.Append("0.000 0.000 0.000 1.000 k" + _newline);
				strOut.Append(((Cx * ModuleSizeX + OffsetX + Ray).ToString() + " " + (Cy * ModuleSizeY + OffsetY).ToString() + " m").Replace(",", ".") + NewLine);

				strOut.Append(((Cx * ModuleSizeX + OffsetX + Ray).ToString() + " " + (Cy * ModuleSizeY + OffsetY - sngL).ToString() + " " +
								 (Cx * ModuleSizeX + OffsetX + sngL).ToString() + " " + (Cy * ModuleSizeY + OffsetY - Ray).ToString() + " " +
								 (Cx * ModuleSizeX + OffsetX).ToString() + " " + (Cy * ModuleSizeY + OffsetY - Ray).ToString() + " c").Replace(",", ".") + NewLine);

				strOut.Append(((Cx * ModuleSizeX + OffsetX - sngL).ToString() + " " + (Cy * ModuleSizeY + OffsetY - Ray).ToString() + " " +
								 (Cx * ModuleSizeX + OffsetX - Ray).ToString() + " " + (Cy * ModuleSizeY + OffsetY - sngL).ToString() + " " +
								 (Cx * ModuleSizeX + OffsetX - Ray).ToString() + " " + (Cy * ModuleSizeY + OffsetY).ToString() + " c").Replace(",", ".") + NewLine);

				strOut.Append(((Cx * ModuleSizeX + OffsetX - Ray).ToString() + " " + (Cy * ModuleSizeY + OffsetY + sngL).ToString() + " " +
								 (Cx * ModuleSizeX + OffsetX - sngL).ToString() + " " + (Cy * ModuleSizeY + OffsetY + Ray).ToString() + " " +
								 (Cx * ModuleSizeX + OffsetX).ToString() + " " + (Cy * ModuleSizeY + OffsetY + Ray).ToString() + " c").Replace(",", ".") + NewLine);

				strOut.Append(((Cx * ModuleSizeX + OffsetX + sngL).ToString() + " " + (Cy * ModuleSizeY + OffsetY + Ray).ToString() + " " +
								 (Cx * ModuleSizeX + OffsetX + Ray).ToString() + " " + (Cy * ModuleSizeY + OffsetY + sngL).ToString() + " " +
								 (Cx * ModuleSizeX + OffsetX + Ray).ToString() + " " + (Cy * ModuleSizeY + OffsetY).ToString() + " c").Replace(",", ".") + NewLine);
				strOut.Append("f" + NewLine);


				Ray = 0.57d * ModuleSizeX ;
				sngL = (double)(4d / 3d * (Math.Sqrt(2d) - 1d) * Ray);
				strOut.Append("0.000 0.000 0.000 0.000 k" + _newline);
				strOut.Append(((Cx * ModuleSizeX + OffsetX + Ray).ToString() + " " + (Cy * ModuleSizeY + OffsetY).ToString() + " m").Replace(",", ".") + NewLine);

				strOut.Append(((Cx * ModuleSizeX + OffsetX + Ray).ToString() + " " + (Cy * ModuleSizeY + OffsetY - sngL).ToString() + " " +
								 (Cx * ModuleSizeX + OffsetX + sngL).ToString() + " " + (Cy * ModuleSizeY + OffsetY - Ray).ToString() + " " +
								 (Cx * ModuleSizeX + OffsetX).ToString() + " " + (Cy * ModuleSizeY + OffsetY - Ray).ToString() + " c").Replace(",", ".") + NewLine);

				strOut.Append(((Cx * ModuleSizeX + OffsetX - sngL).ToString() + " " + (Cy * ModuleSizeY + OffsetY - Ray).ToString() + " " +
								 (Cx * ModuleSizeX + OffsetX - Ray).ToString() + " " + (Cy * ModuleSizeY + OffsetY - sngL).ToString() + " " +
								 (Cx * ModuleSizeX + OffsetX - Ray).ToString() + " " + (Cy * ModuleSizeY + OffsetY).ToString() + " c").Replace(",", ".") + NewLine);

				strOut.Append(((Cx * ModuleSizeX + OffsetX - Ray).ToString() + " " + (Cy * ModuleSizeY + OffsetY + sngL).ToString() + " " +
								 (Cx * ModuleSizeX + OffsetX - sngL).ToString() + " " + (Cy * ModuleSizeY + OffsetY + Ray).ToString() + " " +
								 (Cx * ModuleSizeX + OffsetX).ToString() + " " + (Cy * ModuleSizeY + OffsetY + Ray).ToString() + " c").Replace(",", ".") + NewLine);

				strOut.Append(((Cx * ModuleSizeX + OffsetX + sngL).ToString() + " " + (Cy * ModuleSizeY + OffsetY + Ray).ToString() + " " +
								 (Cx * ModuleSizeX + OffsetX + Ray).ToString() + " " + (Cy * ModuleSizeY + OffsetY + sngL).ToString() + " " +
								 (Cx * ModuleSizeX + OffsetX + Ray).ToString() + " " + (Cy * ModuleSizeY + OffsetY).ToString() + " c").Replace(",", ".") + NewLine);
				strOut.Append("f" + NewLine);


				strOut.Append("*U" + _newline);
                strOut.Append("%%Trailer" + _newline);
                strOut.Append("%%EOF" + _newline);

            }
            else if (VectorType == VectorTypes.VecHTML)
			{
				strOut.Append("<meta http-equiv='X-UA-Compatible' content='IE=10' />");
				strOut.Append("<div style='font-size:0;background-color:#ffffff;position:relative;width:" + tempW + "px;height:" + tempH + "px;'>" + _newline);

				for (int y = 0; y < rows; y++)
				{
					for (int x = 0; x < cols; x++)
					{
						if ((y & 1) != 0)
							OffsetX = ModuleSizeX;
						else
							OffsetX = ModuleSizeX * 0.5;

						if (Matrix[y][x] == 1)
						{

							strOut.Append("<div style='font-size:" + (1.3d*ModuleSizeX ).ToString().Replace(",", ".") + "px;color:#000000;position:absolute;left:" +
										  ((x + QuietZone) * ModuleSizeX + OffsetX).ToString().Replace(",", ".") + "px;top:" + 
										  ((y + QuietZone) * ModuleSizeY + OffsetY).ToString().Replace(",", ".") + "px;'>&#x2B22;</div>" + _newline);
						}
					}
				}

				int cx = 14 + QuietZone;
				int cy = 16 + QuietZone;
				OffsetX = ModuleSizeX * 1d;
				OffsetY = ModuleSizeY * 1.5d;
				Ray = 0.58d * ModuleSizeX;
				strOut.Append("<div style='-webkit-border-radius:50%; -moz-border-radius:50%; border-radius:50%;" + 
					"background:#000000;" + 
					"width:" + (2 * Ray * 7.59).ToString().Replace(",", ".") + "px;" + 
					"height:" + (2 * Ray * 7.59).ToString().Replace(",", ".") + "px;" +
					"position:absolute;left:" + (cx * ModuleSizeX + OffsetX - Ray * 7.59).ToString().Replace(",", ".") + "px;" +
					"top:" + (cy * ModuleSizeY + OffsetY - Ray * 7.59).ToString().Replace(",", ".") + "px;'>&nbsp</div>" + NewLine);
				strOut.Append("<div style='-webkit-border-radius:50%; -moz-border-radius:50%; border-radius:50%;" +
					"background:#ffffff;" +
					"width:" + (2 * Ray * 6.27).ToString().Replace(",", ".") + "px;" +
					"height:" + (2 * Ray * 6.27).ToString().Replace(",", ".") + "px;" +
					"position:absolute;left:" + (cx * ModuleSizeX + OffsetX - Ray * 6.27).ToString().Replace(",", ".") + "px;" +
					"top:" + (cy * ModuleSizeY + OffsetY - Ray * 6.27).ToString().Replace(",", ".") + "px;'>&nbsp</div>" + NewLine);
				strOut.Append("<div style='-webkit-border-radius:50%; -moz-border-radius:50%; border-radius:50%;" +
					"background:#000000;" +
					"width:" + (2 * Ray * 4.96).ToString().Replace(",", ".") + "px;" +
					"height:" + (2 * Ray * 4.96).ToString().Replace(",", ".") + "px;" +
					"position:absolute;left:" + (cx * ModuleSizeX + OffsetX - Ray * 4.96).ToString().Replace(",", ".") + "px;" +
					"top:" + (cy * ModuleSizeY + OffsetY - Ray * 4.96).ToString().Replace(",", ".") + "px;'>&nbsp</div>" + NewLine);
				strOut.Append("<div style='-webkit-border-radius:50%; -moz-border-radius:50%; border-radius:50%;" +
					"background:#ffffff;" +
					"width:" + (2 * Ray * 3.64).ToString().Replace(",", ".") + "px;" +
					"height:" + (2 * Ray * 3.64).ToString().Replace(",", ".") + "px;" +
					"position:absolute;left:" + (cx * ModuleSizeX + OffsetX - Ray * 3.64).ToString().Replace(",", ".") + "px;" +
					"top:" + (cy * ModuleSizeY + OffsetY - Ray * 3.64).ToString().Replace(",", ".") + "px;'>&nbsp</div>" + NewLine);
				strOut.Append("<div style='-webkit-border-radius:50%; -moz-border-radius:50%; border-radius:50%;" +
					"background:#000000;" +
					"width:" + (2 * Ray * 2.31).ToString().Replace(",", ".") + "px;" +
					"height:" + (2 * Ray * 2.31).ToString().Replace(",", ".") + "px;" +
					"position:absolute;left:" + (cx * ModuleSizeX + OffsetX - Ray * 2.31).ToString().Replace(",", ".") + "px;" +
					"top:" + (cy * ModuleSizeY + OffsetY - Ray * 2.31).ToString().Replace(",", ".") + "px;'>&nbsp</div>" + NewLine);
				strOut.Append("<div style='-webkit-border-radius:50%; -moz-border-radius:50%; border-radius:50%;" +
					"background:#ffffff;" +
					"width:" + (2 * Ray).ToString().Replace(",", ".") + "px;" +
					"height:" + (2 * Ray).ToString().Replace(",", ".") + "px;" +
					"position:absolute;left:" + (cx * ModuleSizeX + OffsetX - Ray).ToString().Replace(",", ".") + "px;" +
					"top:" + (cy * ModuleSizeY + OffsetY - Ray).ToString().Replace(",", ".") + "px;'>&nbsp</div>" + NewLine);

				strOut.Append("</div>" + _newline);
			}

			return strOut.ToString();

		}



		/// <summary>
		/// Function, which creates a MaxiCode barcode, depending on the various input options chosen. 
		/// </summary>
		/// <param name="DataToEncode">The data that is encoded in the barcode.</param>
		/// <param name="ApplyTilde">Enables processing of tilde and parentheses characters for purposes of encoding ASCII functions and GS1 application identifiers.</param>
		/// <param name="EncodingMode">Various modes used to Data represented in the symbol may be encoded by one of these types.</param>
		/// <param name="NewLine">String indicating EOL character. Defaults to the operating system EOL character if left NULL.</param>
		/// <returns>Encoded text for use with the 2D barcode font.</returns>
		public string EncodeMaxiCode(string DataToEncode, bool ApplyTilde = true, EncodingModes EncMode = EncodingModes.Mode2, string NewLine = null)
		{
			encodingMode = EncMode;
			//allows user to set an end-of-line (EOL) string
			if (NewLine == null)
			{
				NewLine = Environment.NewLine; //default
			}

			_newline = NewLine;

			_encode(DataToEncode, ApplyTilde);

			StringBuilder StringToPrint = new StringBuilder();
			try
			{

				for (int row = 0; row < 33; row = row + 6)
				{
					for (int col = 0; col < 30; col++)
					{

						if (col == 14 && row == 12)
							StringToPrint.Append("V");
						else if (col == 14 && row == 18)
							StringToPrint.Append("X");

						else if (Matrix[row][col] == 0)
						{

							if (Matrix[row + 1][col] == 0)
							{
								if (Matrix[row + 2][ col] == 0)
									StringToPrint.Append("0");
								else
									StringToPrint.Append("1");
							}
							else
							{

								if (Matrix[row + 2][col] == 0)
									StringToPrint.Append("2");
								else
									StringToPrint.Append("3");
							}
						}
						else
						{
							if (Matrix[row + 1][col] == 0)
							{
								if (Matrix[row + 2][col] == 0)
									StringToPrint.Append("4");
								else
									StringToPrint.Append("5");
							}
							else
							{
								if (Matrix[row + 2][col] == 0)
									StringToPrint.Append("6");
								else
									StringToPrint.Append("7");
							}
						}

					}
					StringToPrint.Append(_newline);

					if ((row + 4) < 33) { 
						for (int col = 0; col < 30; col++)
						{

							if (col == 14 && row + 3 == 15)
								StringToPrint.Append("W");
							else if (Matrix[row + 3][col] == 0)
							{

								if (Matrix[row + 4][col] == 0)
								{
									if (Matrix[row + 5][col] == 0)
										StringToPrint.Append("0");
									else if (Matrix[row + 5][col] == -2)
										StringToPrint.Append("E");
									else
										StringToPrint.Append("N");
								}
								else
								{

									if (Matrix[row + 5][col] == 0)
										StringToPrint.Append("O");
									else if (Matrix[row + 5][col] == -2)
										StringToPrint.Append("F");
									else
										StringToPrint.Append("P");
								}
							}
							else if (Matrix[row + 3][col] == -3)
							{

								if (Matrix[row + 4][col] == 0)
								{
									if (Matrix[row + 5][col] == 0)
										StringToPrint.Append("p");
									else
										StringToPrint.Append("q");
								}
								else
								{
									if (Matrix[row + 5][col] == 0)
										StringToPrint.Append("r");
									else
										StringToPrint.Append("s");
								}


							}
							else
							{
								if (Matrix[row + 4][col] == 0)
								{
									if (Matrix[row + 5][col] == 0)
										StringToPrint.Append("Q");
									else if (Matrix[row + 5][col] == -2)
										StringToPrint.Append("G");
									else
										StringToPrint.Append("R");
								}
								else
								{
									if (Matrix[row + 5][col] == 0)
										StringToPrint.Append("S");
									else if (Matrix[row + 5][col] == -2)
										StringToPrint.Append("H");
									else
										StringToPrint.Append("T");
								}
							}

						}
						StringToPrint.Append(_newline);
					}
				}

			}
			catch (Exception localException)
			{
				System.Console.Out.WriteLine("Error in EncodeMaxiCode " + localException.ToString());
			}
			return StringToPrint.ToString();
		}


		#region Internal and Private variables

		/// <summary>
		/// This is a standard value for newline
		/// </summary>

		private String _newline = "";

		/// <summary>
		/// This is a matrix values that we will draw the code on
		/// </summary>
		internal int[][] Matrix = null;

		/// <summary>
		/// This is a layout mask that we will draw the hexagons on
		/// </summary>
		internal int[][] layout = null;
		/// <summary>
		/// The data to be encoded
		/// </summary>
		internal string UserData = "IDAutomation.com, Inc.";

		/// <summary>
		/// The type of encoding that should be done.  Based on the Maxicode standards
		/// </summary>
		internal EncodingModes encodingMode = EncodingModes.Mode2;

		/// <summary>
		/// A 3 digit number representing the Ship To Country Code
		/// </summary>
		private string countryCode = "";

		/// <summary>
		/// A 3 digit number representing the Class of Service
		/// </summary>
		private string classOfService = "";

		/// <summary>
		/// A 5 or 9 digit number representing the zip code
		/// </summary>
		internal string zipCode = "";

		///<summary>
		///Used for Structured Append:  Number of Symbol in structured appended mode
		///</summary>
		internal int numberOfCodes = 1;
		
		///<summary>
		///Used for Structured Append:  Position of this symbol in the sequence of the structured appended
		///</summary>
		internal int positionOfCode = 1;
		#endregion

		#region Mehtods
		/// <summary>
		/// This is the driver for encoding Maxicode data into a format for barcoding
		/// </summary>
		internal void _encode(string DataToBarcode, bool ApplyTilde = true)
		{

			// Matrix[Row,Column]
			//			Matrix = new int[33, 30];

			Matrix = new int[][] {
				new int [] { 1, 1, 0, 1, 0, 1, 1, 1, 0, 1, 1, 0, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 1, 1, 1 },
				new int [] { 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 1, 1, 0, 0, 0, 0, 1, 1, 0, 1, 1, 0, 0, 0, 0, 1, 0, 0, 0, 0},
				new int [] { 1, 1, 0, 1, 1, 0, 1, 1, 0, 0, 0, 1, 0, 0, 1, 0, 1, 0, 1, 0, 0, 1, 1, 0, 0, 1, 1, 0, 0, 1},
				new int [] { 1, 1, 1, 0, 0, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0},
				new int [] { 1, 0, 1, 1, 1, 1, 0, 0, 0, 0, 1, 0, 1, 1, 0, 1, 0, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 1},
				new int [] { 0, 0, 1, 0, 0, 1, 1, 1, 0, 0, 1, 0, 1, 0, 0, 0, 1, 1, 0, 0, 1, 1, 0, 0, 0, 1, 0, 0, 1, 0},
				new int [] { 1, 1, 1, 1, 0, 0, 1, 1, 1, 0, 1, 0, 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 1, 0, 1, 1, 1, 0, 1},
				new int [] { 0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 1, 1, 0, 0, 0, 1, 1, 0, 1, 1, 1, 1, 0, 0, 1, 1, 0, 0, 1, 0},
				new int [] { 1, 0, 0, 0, 1, 0, 1, 0, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 1, 1, 0, 0, 0},
				new int [] { 1, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 1, 1, 0, 0, 0, 0, 0, 0, 1, 0, 1, 1, 0, 1, 0, 1, 0, 0, 0},
				new int [] { 1, 0, 0, 0, 0, 0, 1, 1, 0, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 0, 0},
				new int [] { 1, 0, 1, 1, 1, 0, 1, 0, 1, 0, 0, 0, 0, 0, -2, 0, 0, 1, 0, 1, 0, 1, 1, 0, 1, 0, 1, 0, 0, 0},
				new int [] { 1, 0, 1, 1, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 1, 0, 1, 1, 1, 0},
				new int [] { 0, 0, 1, 1, 1, 0, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 1, 1, 1, 1, 1, 1, 0},
				new int [] { 0, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 1, 1, 1, 0, 1, 1, 0, 0},
				new int [] { 1, 0, 0, 0, 1, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 0, 0, 0, 0, 1, 0, 1, 0},
				new int [] { 1, 1, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 1, 0, 0, 1, 0, 1, 0},
				new int [] { 1, 0, 1, 0, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 0, 1, 0, 0, 0, 0, 0},
				new int [] { 0, 0, 0, 0, 1, 0, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 1, 0, 0, 1, 1, 1, 0},
				new int [] { 1, 0, 1, 1, 1, 1, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 0, 1, 1, 1, 0},
				new int [] { 0, 1, 0, 0, 0, 1, 1, 1, 0, 1, 1, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 1, 0, 1, 0, 0, 1, 1, 1, 0},
				new int []{ 0, 1, 0, 1, 0, 1, 0, 1, 1, 1, 0, 0, 0, 0, -3, 0, 0, 0, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 1, 0},
				new int []{ 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 0, 0, 1, 1, 0, 1, 1, 0, 1, 1, 1, 0, 0, 0, 0, 0, 0, 1, 1},
				new int [] { 1, 0, 1, 0, 1, 0, 1, 0, 0, 1, 1, 1, 1, 1, 0, 0, 0, 1, 1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0},
				new int [] { 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 1, 1, 0, 1, 1, 1, 1, 1, 0, 1},
				new int [] { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 0, 0, 0},
				new int [] { 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 0, 0, 1, 0, 1, 1, 1, 1},
				new int [] { 0, 1, 0, 0, 1, 1, 0, 1, 0, 0, 0, 1, 0, 0, 1, 0, 0, 1, 1, 1, 0, 1, 1, 0, 0, 0, 1, 0, 0, 0},
				new int [] { 0, 1, 0, 0, 1, 1, 0, 1, 1, 0, 1, 0, 1, 1, 1, 0, 0, 0, 1, 1, 0, 1, 1, 1, 1, 0, 0, 0, 0, 1},
				new int [] { 1, 1, 1, 1, 0, 1, 1, 1, 1, 0, 0, 1, 0, 0, 0, 1, 1, 1, 0, 1, 0, 0, 0, 1, 0, 0, 1, 0, 0, 0},
				new int [] { 0, 0, 1, 0, 1, 0, 0, 1, 1, 0, 0, 0, 1, 1, 0, 1, 1, 0, 1, 1, 1, 0, 0, 1, 1, 1, 1, 0, 1, 1},
				new int [] { 1, 0, 0, 0, 1, 1, 0, 0, 1, 0, 1, 1, 1, 0, 0, 0, 1, 1, 0, 0, 1, 1, 0, 0, 1, 1, 1, 0, 1, 0},
				new int []{ 1, 0, 1, 0, 0, 1, 0, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 0, 0, 0, 1, 1, 0, 1, 0}
			};
			return;
		}//End _encode


	}


}

//ENUMERATIONS:

/// <summary>
/// The mode of compression and encoding used in the symbol.
/// </summary>
public enum EncodingModes
{
	/// <summary>
	/// Modes 2 is designed for use in the transport industry.
	/// </summary>
	Mode2,
	/// <summary>
	/// Modes 3 is designed for use in the transport industry.
	/// </summary>
	Mode3,
	/// <summary>
	/// Mode 4 encodes up to 93 characters or 138 digits.
	/// </summary>
	Mode4,
	/// <summary>
	/// Mode 5 encodes up to 77 characters and provides more error correction capabilities than mode 4.
	/// </summary>
	Mode5,
	/// <summary>
	/// Mode 6 indicates that the symbol encodes a message used to program a reader system (scanner).
	/// </summary>
	Mode6
}

/// <summary>
/// Various types of vector image output.
/// </summary>
public enum VectorTypes
{

	/// <summary>
	/// Returns a string with the SVG rapresentation. 
	/// </summary>
	VecSVG,

	/// <summary>
	/// Return a string with the EPS rapresentation.
	/// </summary>
	VecEPS,

	/// <summary>
	/// Return a string with the HTML rapresentation.
	/// </summary>
	VecHTML,

}

#endregion